"""
File const.py contains most constants from SLMP documentation.

Needs to be called with .value parameter.
E.g:
    from const import FrameType

    x = FrameType.SLMP_FTYPE_BIN_REQ_ST.value
"""
import enum

MAX_FRAME_SIZE = 1518  # Maximal length of request


class FTHeaderSize(enum.Enum):
    """
    Length of necessary header of message for every type of frame.
    """
    BIN_REQ_ST_SIZE = 6
    BIN_RES_ST_SIZE = 2
    BIN_REQ_MT_SIZE = 6
    BIN_RES_MT_SIZE = 2
    ASCII_REQ_ST_SIZE = 12
    ASCII_RES_ST_SIZE = 4
    ASCII_REQ_MT_SIZE = 12
    ASCII_RES_MT_SIZE = 4


class FrameType(enum.Enum):
    """
    Definition of Frame Type.
    Whether message has ASCII or BINARY form,
    is REQUEST or RESPONSE (ST)
    and is SINGLE TRANSMISSION or MULTIPLE TRANSMISSION (MT).
    """
    SLMP_FTYPE_BIN_REQ_ST = 0x5000
    SLMP_FTYPE_BIN_RES_ST = 0xD000
    SLMP_FTYPE_BIN_REQ_MT = 0x5400
    SLMP_FTYPE_BIN_RES_MT = 0xD400
    SLMP_FTYPE_ASCII_REQ_ST = 0x35303030  # '5000'
    SLMP_FTYPE_ASCII_RES_ST = 0x44303030  # 'D000'
    SLMP_FTYPE_ASCII_REQ_MT = 0x35343030  # '5400'
    SLMP_FTYPE_ASCII_RES_MT = 0x44343030  # 'D400'


class SLMPSubCommand(enum.Enum):
    """
    Subcommand telling whether we communicate with registers with binary/word/double word data or whether
    we communicate with CPU.
    """
    SUB_bit1 = 0x0001
    SUB_bit3 = 0x0003
    SUB_word0 = 0x0000
    SUB_word2 = 0x0002
    SUB_cpu0 = 0x0080
    SUB_cpu2 = 0x0082


class SLMPCommand(enum.Enum):
    """
    List of SLMP Commands.
    """
    # Device
    SLMP_COMMAND_DEVICE_READ = 0x0401
    SLMP_COMMAND_DEVICE_WRITE = 0x1401
    SLMP_COMMAND_DEVICE_READ_RANDOM = 0x0403
    SLMP_COMMAND_DEVICE_WRITE_RANDOM = 0x1402
    SLMP_COMMAND_DEVICE_ENTRY_MONITOR_DEVICE = 0x0801
    SLMP_COMMAND_DEVICE_EXECUTE_MONITOR = 0x0802
    SLMP_COMMAND_DEVICE_READ_BLOCK = 0x0406
    SLMP_COMMAND_DEVICE_WRITE_BLOCK = 0x1406

    # Memory
    SLMP_COMMAND_MEMORY_READ = 0x0613
    SLMP_COMMAND_MEMORY_WRITE = 0x1613

    # ExtendUnit
    SLMP_COMMAND_EXTEND_UNIT_READ = 0x0601
    SLMP_COMMAND_EXTEND_UNIT_WRITE = 0x1601

    # RemoteControl
    SLMP_COMMAND_REMOTE_RUN = 0x1001
    SLMP_COMMAND_REMOTE_STOP = 0x1002
    SLMP_COMMAND_REMOTE_PAUSE = 0x1003
    SLMP_COMMAND_REMOTE_LATCH_CLEAR = 0x1005
    SLMP_COMMAND_REMOTE_RESET = 0x1006
    SLMP_COMMAND_REMOTE_READ_TYPE_NAME = 0x0101

    # Drive
    SLMP_COMMAND_DRIVE_READ_DISK_STATE = 0x0205
    SLMP_COMMAND_DRIVE_DEFRAG = 0x1207

    # File
    SLMP_COMMAND_FILE_READ_FILE_INFO = 0x0201
    SLMP_COMMAND_FILE_READ_FILE_INFO_DETAIL = 0x0202
    SLMP_COMMAND_FILE_READ_FILE_INFO_FILE_NUMBER_USAGE = 0x0204
    SLMP_COMMAND_FILE_CHANGE_FILE_INFO = 0x1204
    SLMP_COMMAND_FILE_SEARCH = 0x0203
    SLMP_COMMAND_FILE_READ_ACCESS_TYPE_A = 0x0206
    SLMP_COMMAND_FILE_WRITE_ACCESS_TYPE_A = 0x1203
    SLMP_COMMAND_FILE_LOCK_CONTROL = 0x0808
    SLMP_COMMAND_FILE_COPY_ACCESS_TYPE_A = 0x1206
    SLMP_COMMAND_FILE_COPY_ACCESS_TYPE_B = 0x1824
    SLMP_COMMAND_FILE_DELETE_ACCESS_TYPE_A = 0x1205
    SLMP_COMMAND_FILE_DELETE_ACCESS_TYPE_B = 0x1822
    SLMP_COMMAND_FILE_READ_DIRECTORY_FILE = 0x1810
    SLMP_COMMAND_FILE_SEARCH_DIRECTORY_FILE = 0x1811
    SLMP_COMMAND_FILE_CREATE_NEW_FILE_ACCESS_TYPE_A = 0x1202
    SLMP_COMMAND_FILE_CREATE_NEW_FILE_ACCESS_TYPE_B = 0x1820
    SLMP_COMMAND_FILE_CHANGE_FILE_STATE = 0x1825
    SLMP_COMMAND_FILE_CHANGE_FILE_DATE = 0x1826
    SLMP_COMMAND_FILE_OPEN_FILE = 0x1827
    SLMP_COMMAND_FILE_READ_ACCESS_TYPE_B = 0x1828
    SLMP_COMMAND_FILE_WRITE_ACCESS_TYPE_B = 0x1829
    SLMP_COMMAND_FILE_CLOSE_FILE = 0x182A

    # Test
    SLMP_COMMAND_SELF_TEST = 0x0619

    # ClearError
    SLMP_COMMAND_CLEAR_ERROR_CODE = 0x1617

    # RemotePassword
    SLMP_COMMAND_PASSWORD_LOCK = 0x1630
    SLMP_COMMAND_PASSWORD_UNLOCK = 0x1631

    # OnDemand
    SLMP_COMMAND_ONDEMAND = 0x2101

    # NodeConnect
    SLMP_COMMAND_NODE_SEARCH = 0x0E30
    SLMP_COMMAND_IP_ADDRESS_SET = 0x0E31

    # ParameterSetting
    SLMP_COMMAND_DEVICE_INFO_COMPARE = 0x0E32
    SLMP_COMMAND_PARAMETER_GET = 0x0E33
    SLMP_COMMAND_PARAMETER_SET = 0x0E34
    SLMP_COMMAND_PARAMETER_SET_START = 0x0E35
    SLMP_COMMAND_PARAMETER_SET_END = 0x0E36
    SLMP_COMMAND_PARAMETER_SET_CANCEL = 0x0E3A

    # NodeMonitoring
    SLMP_COMMAND_STATUS_READ = 0x0E44
    SLMP_COMMAND_COMMUNICATION_SETTING_GET = 0x0E45
    SLMP_COMMAND_STATUS_READ2 = 0x0E53


class ProcessorNumber(enum.Enum):
    """
    Definition of Processor Number or as documentation refer Destination unit I/O No.
    """
    SLMP_CPU_ACTIVE = 0x03D0
    SLMP_CPU_STANDBY = 0x03D1
    SLMP_CPU_TYPE_A = 0x03D2
    SLMP_CPU_TYPE_B = 0x03D3
    SLMP_CPU_1 = 0x03E0
    SLMP_CPU_2 = 0x03E1
    SLMP_CPU_3 = 0x03E2
    SLMP_CPU_4 = 0x03E3
    SLMP_CPU_DEFAULT = 0x03FF


class TimerValue(enum.Enum):
    """
    Definition of Monitoring timer value.
    Can be increased, check documentation.
    """
    SLMP_TIMER_WAIT_FOREVER = 0x0000


class ErrCode(enum.Enum):
    """
    Definition of SLMP Error Codes
    """
    SLMP_ERR_COMMAND_SUBCOMMAND = 0xC059
    SLMP_ERR_WRONG_DATA = 0xC05C
    SLMP_ERR_DATA_LENGTH = 0xC061
    SLMP_ERR_UNDER_EXECUTION = 0xCEE0
    SLMP_ERR_REQ_DATA_SIZE = 0xCEE1
    SLMP_ERR_RES_DATA_SIZE = 0xCEE2
    SLMP_ERR_NO_EXIST_SERVER_NO = 0xCF10
    SLMP_ERR_CAN_NOT_COMMUNICATION_SETTING = 0xCF20
    SLMP_ERR_NO_EXIST_PARAM_ID = 0xCF30
    SLMP_ERR_CAN_NOT_PARAMETER_SET = 0xCF31
    SLMP_ERR_OK = 0
    SLMP_ERR_NG = -1


class FrameIndex(enum.Enum):
    """
    Definition of starting data index per frame type in response.
    """
    SLMP_FTYPE_BIN_REQ_ST_INDEX = 0x00
    SLMP_FTYPE_BIN_RES_ST_INDEX = 0x01
    SLMP_FTYPE_BIN_REQ_MT_INDEX = 0x02
    SLMP_FTYPE_BIN_RES_MT_INDEX = 0x03
    SLMP_FTYPE_ASCII_REQ_ST_INDEX = 0x04
    SLMP_FTYPE_ASCII_RES_ST_INDEX = 0x05
    SLMP_FTYPE_ASCII_REQ_MT_INDEX = 0x06
    SLMP_FTYPE_ASCII_RES_MT_INDEX = 0x07
